@@ -21,15 +21,15 @@ import (
 	"sync"
 	"time"
 
-	"github.com/go-kit/kit/log"
-	"github.com/go-kit/kit/log/level"
+	"github.com/go-kit/log"
+	"github.com/go-kit/log/level"
+	"github.com/go-openapi/analysis"
 	"github.com/go-openapi/loads"
 	"github.com/go-openapi/runtime/middleware"
 	"github.com/go-openapi/strfmt"
 	"github.com/prometheus/client_golang/prometheus"
 	prometheus_model "github.com/prometheus/common/model"
 	"github.com/prometheus/common/version"
-	"github.com/prometheus/prometheus/pkg/labels"
 	"github.com/rs/cors"
 
 	"github.com/prometheus/alertmanager/api/metrics"
@@ -44,7 +44,7 @@ import (
 	"github.com/prometheus/alertmanager/cluster"
 	"github.com/prometheus/alertmanager/config"
 	"github.com/prometheus/alertmanager/dispatch"
-	"github.com/prometheus/alertmanager/pkg/parse"
+	"github.com/prometheus/alertmanager/pkg/labels"
 	"github.com/prometheus/alertmanager/provider"
 	"github.com/prometheus/alertmanager/silence"
 	"github.com/prometheus/alertmanager/silence/silencepb"
@@ -53,7 +53,7 @@ import (
 
 // API represents an Alertmanager API v2
 type API struct {
-	peer           *cluster.Peer
+	peer           cluster.ClusterPeer
 	silences       *silence.Silences
 	alerts         provider.Alerts
 	alertGroups    groupsFn
@@ -74,9 +74,11 @@ type API struct {
 	Handler http.Handler
 }
 
-type groupsFn func(func(*dispatch.Route) bool, func(*types.Alert, time.Time) bool) (dispatch.AlertGroups, map[prometheus_model.Fingerprint][]string)
-type getAlertStatusFn func(prometheus_model.Fingerprint) types.AlertStatus
-type setAlertStatusFn func(prometheus_model.LabelSet)
+type (
+	groupsFn         func(func(*dispatch.Route) bool, func(*types.Alert, time.Time) bool) (dispatch.AlertGroups, map[prometheus_model.Fingerprint][]string)
+	getAlertStatusFn func(prometheus_model.Fingerprint) types.AlertStatus
+	setAlertStatusFn func(prometheus_model.LabelSet)
+)
 
 // NewAPI returns a new Alertmanager API v2
 func NewAPI(
@@ -84,7 +86,7 @@ func NewAPI(
 	gf groupsFn,
 	sf getAlertStatusFn,
 	silences *silence.Silences,
-	peer *cluster.Peer,
+	peer cluster.ClusterPeer,
 	l log.Logger,
 	r prometheus.Registerer,
 ) (*API, error) {
@@ -99,20 +101,22 @@ func NewAPI(
 		uptime:         time.Now(),
 	}
 
-	// load embedded swagger file
-	swaggerSpec, err := loads.Analyzed(restapi.SwaggerJSON, "")
+	// Load embedded swagger file.
+	swaggerSpec, swaggerSpecAnalysis, err := getSwaggerSpec()
 	if err != nil {
-		return nil, fmt.Errorf("failed to load embedded swagger file: %v", err.Error())
+		return nil, err
 	}
 
-	// create new service API
+	// Create new service API.
 	openAPI := operations.NewAlertmanagerAPI(swaggerSpec)
 
 	// Skip the  redoc middleware, only serving the OpenAPI specification and
 	// the API itself via RoutesHandler. See:
 	// https://github.com/go-swagger/go-swagger/issues/1779
 	openAPI.Middleware = func(b middleware.Builder) http.Handler {
-		return middleware.Spec("", swaggerSpec.Raw(), openAPI.Context().RoutesHandler(b))
+		// Manually create the context so that we can use the singleton swaggerSpecAnalysis.
+		swaggerContext := middleware.NewRoutableContextWithAnalyzedSpec(swaggerSpec, swaggerSpecAnalysis, openAPI, nil)
+		return middleware.Spec("", swaggerSpec.Raw(), swaggerContext.RoutesHandler(b))
 	}
 
 	openAPI.AlertGetAlertsHandler = alert_ops.GetAlertsHandlerFunc(api.getAlertsHandler)
@@ -125,14 +129,29 @@ func NewAPI(
 	openAPI.SilenceGetSilencesHandler = silence_ops.GetSilencesHandlerFunc(api.getSilencesHandler)
 	openAPI.SilencePostSilencesHandler = silence_ops.PostSilencesHandlerFunc(api.postSilencesHandler)
 
-	openAPI.Logger = func(s string, i ...interface{}) { level.Error(api.logger).Log(i...) }
-
 	handleCORS := cors.Default().Handler
-	api.Handler = handleCORS(openAPI.Serve(nil))
+	api.Handler = handleCORS(setResponseHeaders(openAPI.Serve(nil)))
 
 	return &api, nil
 }
 
+var responseHeaders = map[string]string{
+	"Cache-Control": "no-store",
+}
+
+func setResponseHeaders(h http.Handler) http.Handler {
+	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		for h, v := range responseHeaders {
+			w.Header().Set(h, v)
+		}
+		h.ServeHTTP(w, r)
+	})
+}
+
+func (api *API) requestLogger(req *http.Request) log.Logger {
+	return log.With(api.logger, "path", req.URL.Path, "method", req.Method)
+}
+
 // Update sets the API struct members that may change between reloads of alertmanager.
 func (api *API) Update(cfg *config.Config, setAlertStatus setAlertStatusFn) {
 	api.mtx.Lock()
@@ -167,6 +186,7 @@ func (api *API) getStatusHandler(params general_ops.GetStatusParams) middleware.
 		},
 		Cluster: &open_api_models.ClusterStatus{
 			Status: &status,
+			Peers:  []*open_api_models.PeerStatus{},
 		},
 	}
 
@@ -177,8 +197,9 @@ func (api *API) getStatusHandler(params general_ops.GetStatusParams) middleware.
 		peers := []*open_api_models.PeerStatus{}
 		for _, n := range api.peer.Peers() {
 			address := n.Address()
+			name := n.Name()
 			peers = append(peers, &open_api_models.PeerStatus{
-				Name:    &n.Name,
+				Name:    &name,
 				Address: &address,
 			})
 		}
@@ -202,8 +223,8 @@ func (api *API) getReceiversHandler(params receiver_ops.GetReceiversParams) midd
 	defer api.mtx.RUnlock()
 
 	receivers := make([]*open_api_models.Receiver, 0, len(api.alertmanagerConfig.Receivers))
-	for _, r := range api.alertmanagerConfig.Receivers {
-		receivers = append(receivers, &open_api_models.Receiver{Name: &r.Name})
+	for i := range api.alertmanagerConfig.Receivers {
+		receivers = append(receivers, &open_api_models.Receiver{Name: &api.alertmanagerConfig.Receivers[i].Name})
 	}
 
 	return receiver_ops.NewGetReceiversOK().WithPayload(receivers)
@@ -216,18 +237,20 @@ func (api *API) getAlertsHandler(params alert_ops.GetAlertsParams) middleware.Re
 		// are no alerts present
 		res = open_api_models.GettableAlerts{}
 		ctx = params.HTTPRequest.Context()
+
+		logger = api.requestLogger(params.HTTPRequest)
 	)
 
 	matchers, err := parseFilter(params.Filter)
 	if err != nil {
-		level.Error(api.logger).Log("msg", "failed to parse matchers", "err", err)
+		level.Debug(logger).Log("msg", "Failed to parse matchers", "err", err)
 		return alertgroup_ops.NewGetAlertGroupsBadRequest().WithPayload(err.Error())
 	}
 
 	if params.Receiver != nil {
 		receiverFilter, err = regexp.Compile("^(?:" + *params.Receiver + ")$")
 		if err != nil {
-			level.Error(api.logger).Log("msg", "failed to compile receiver regex", "err", err)
+			level.Debug(logger).Log("msg", "Failed to compile receiver regex", "err", err)
 			return alert_ops.
 				NewGetAlertsBadRequest().
 				WithPayload(
@@ -265,14 +288,14 @@ func (api *API) getAlertsHandler(params alert_ops.GetAlertsParams) middleware.Re
 			continue
 		}
 
-		alert := alertToOpenAPIAlert(a, api.getAlertStatus(a.Fingerprint()), receivers)
+		alert := AlertToOpenAPIAlert(a, api.getAlertStatus(a.Fingerprint()), receivers)
 
 		res = append(res, alert)
 	}
 	api.mtx.RUnlock()
 
 	if err != nil {
-		level.Error(api.logger).Log("msg", "failed to get alerts", "err", err)
+		level.Error(logger).Log("msg", "Failed to get alerts", "err", err)
 		return alert_ops.NewGetAlertsInternalServerError().WithPayload(err.Error())
 	}
 	sort.Slice(res, func(i, j int) bool {
@@ -283,7 +306,9 @@ func (api *API) getAlertsHandler(params alert_ops.GetAlertsParams) middleware.Re
 }
 
 func (api *API) postAlertsHandler(params alert_ops.PostAlertsParams) middleware.Responder {
-	alerts := openAPIAlertsToAlerts(params.Alerts)
+	logger := api.requestLogger(params.HTTPRequest)
+
+	alerts := OpenAPIAlertsToAlerts(params.Alerts)
 	now := time.Now()
 
 	api.mtx.RLock()
@@ -330,12 +355,12 @@ func (api *API) postAlertsHandler(params alert_ops.PostAlertsParams) middleware.
 		validAlerts = append(validAlerts, a)
 	}
 	if err := api.alerts.Put(validAlerts...); err != nil {
-		level.Error(api.logger).Log("msg", "failed to create alerts", "err", err)
+		level.Error(logger).Log("msg", "Failed to create alerts", "err", err)
 		return alert_ops.NewPostAlertsInternalServerError().WithPayload(err.Error())
 	}
 
 	if validationErrs.Len() > 0 {
-		level.Error(api.logger).Log("msg", "failed to validate alerts", "err", validationErrs.Error())
+		level.Error(logger).Log("msg", "Failed to validate alerts", "err", validationErrs.Error())
 		return alert_ops.NewPostAlertsBadRequest().WithPayload(validationErrs.Error())
 	}
 
@@ -343,18 +368,19 @@ func (api *API) postAlertsHandler(params alert_ops.PostAlertsParams) middleware.
 }
 
 func (api *API) getAlertGroupsHandler(params alertgroup_ops.GetAlertGroupsParams) middleware.Responder {
-	var receiverFilter *regexp.Regexp
+	logger := api.requestLogger(params.HTTPRequest)
 
 	matchers, err := parseFilter(params.Filter)
 	if err != nil {
-		level.Error(api.logger).Log("msg", "failed to parse matchers", "err", err)
+		level.Debug(logger).Log("msg", "Failed to parse matchers", "err", err)
 		return alertgroup_ops.NewGetAlertGroupsBadRequest().WithPayload(err.Error())
 	}
 
+	var receiverFilter *regexp.Regexp
 	if params.Receiver != nil {
 		receiverFilter, err = regexp.Compile("^(?:" + *params.Receiver + ")$")
 		if err != nil {
-			level.Error(api.logger).Log("msg", "failed to compile receiver regex", "err", err)
+			level.Error(logger).Log("msg", "Failed to compile receiver regex", "err", err)
 			return alertgroup_ops.
 				NewGetAlertGroupsBadRequest().
 				WithPayload(
@@ -381,7 +407,7 @@ func (api *API) getAlertGroupsHandler(params alertgroup_ops.GetAlertGroupsParams
 	for _, alertGroup := range alertGroups {
 		ag := &open_api_models.AlertGroup{
 			Receiver: &open_api_models.Receiver{Name: &alertGroup.Receiver},
-			Labels:   modelLabelSetToAPILabelSet(alertGroup.Labels),
+			Labels:   ModelLabelSetToAPILabelSet(alertGroup.Labels),
 			Alerts:   make([]*open_api_models.GettableAlert, 0, len(alertGroup.Alerts)),
 		}
 
@@ -389,7 +415,7 @@ func (api *API) getAlertGroupsHandler(params alertgroup_ops.GetAlertGroupsParams
 			fp := alert.Fingerprint()
 			receivers := allReceivers[fp]
 			status := api.getAlertStatus(fp)
-			apiAlert := alertToOpenAPIAlert(alert, status, receivers)
+			apiAlert := AlertToOpenAPIAlert(alert, status, receivers)
 			ag.Alerts = append(ag.Alerts, apiAlert)
 		}
 		res = append(res, ag)
@@ -426,65 +452,6 @@ func (api *API) alertFilter(matchers []*labels.Matcher, silenced, inhibited, act
 	}
 }
 
-func alertToOpenAPIAlert(alert *types.Alert, status types.AlertStatus, receivers []string) *open_api_models.GettableAlert {
-	startsAt := strfmt.DateTime(alert.StartsAt)
-	updatedAt := strfmt.DateTime(alert.UpdatedAt)
-	endsAt := strfmt.DateTime(alert.EndsAt)
-
-	apiReceivers := make([]*open_api_models.Receiver, 0, len(receivers))
-	for i := range receivers {
-		apiReceivers = append(apiReceivers, &open_api_models.Receiver{Name: &receivers[i]})
-	}
-
-	fp := alert.Fingerprint().String()
-	state := string(status.State)
-	aa := &open_api_models.GettableAlert{
-		Alert: open_api_models.Alert{
-			GeneratorURL: strfmt.URI(alert.GeneratorURL),
-			Labels:       modelLabelSetToAPILabelSet(alert.Labels),
-		},
-		Annotations: modelLabelSetToAPILabelSet(alert.Annotations),
-		StartsAt:    &startsAt,
-		UpdatedAt:   &updatedAt,
-		EndsAt:      &endsAt,
-		Fingerprint: &fp,
-		Receivers:   apiReceivers,
-		Status: &open_api_models.AlertStatus{
-			State:       &state,
-			SilencedBy:  status.SilencedBy,
-			InhibitedBy: status.InhibitedBy,
-		},
-	}
-
-	if aa.Status.SilencedBy == nil {
-		aa.Status.SilencedBy = []string{}
-	}
-
-	if aa.Status.InhibitedBy == nil {
-		aa.Status.InhibitedBy = []string{}
-	}
-
-	return aa
-}
-
-func openAPIAlertsToAlerts(apiAlerts open_api_models.PostableAlerts) []*types.Alert {
-	alerts := []*types.Alert{}
-	for _, apiAlert := range apiAlerts {
-		alert := types.Alert{
-			Alert: prometheus_model.Alert{
-				Labels:       apiLabelSetToModelLabelSet(apiAlert.Labels),
-				Annotations:  apiLabelSetToModelLabelSet(apiAlert.Annotations),
-				StartsAt:     time.Time(apiAlert.StartsAt),
-				EndsAt:       time.Time(apiAlert.EndsAt),
-				GeneratorURL: string(apiAlert.GeneratorURL),
-			},
-		}
-		alerts = append(alerts, &alert)
-	}
-
-	return alerts
-}
-
 func removeEmptyLabels(ls prometheus_model.LabelSet) {
 	for k, v := range ls {
 		if string(v) == "" {
@@ -493,24 +460,6 @@ func removeEmptyLabels(ls prometheus_model.LabelSet) {
 	}
 }
 
-func modelLabelSetToAPILabelSet(modelLabelSet prometheus_model.LabelSet) open_api_models.LabelSet {
-	apiLabelSet := open_api_models.LabelSet{}
-	for key, value := range modelLabelSet {
-		apiLabelSet[string(key)] = string(value)
-	}
-
-	return apiLabelSet
-}
-
-func apiLabelSetToModelLabelSet(apiLabelSet open_api_models.LabelSet) prometheus_model.LabelSet {
-	modelLabelSet := prometheus_model.LabelSet{}
-	for key, value := range apiLabelSet {
-		modelLabelSet[prometheus_model.LabelName(key)] = prometheus_model.LabelValue(value)
-	}
-
-	return modelLabelSet
-}
-
 func receiversMatchFilter(receivers []string, filter *regexp.Regexp) bool {
 	for _, r := range receivers {
 		if filter.MatchString(r) {
@@ -544,7 +493,7 @@ func matchFilterLabels(matchers []*labels.Matcher, sms map[string]string) bool {
 			if m.Value == "" && !prs {
 				continue
 			}
-			if !prs || !m.Matches(v) {
+			if !m.Matches(v) {
 				return false
 			}
 		}
@@ -554,12 +503,14 @@ func matchFilterLabels(matchers []*labels.Matcher, sms map[string]string) bool {
 }
 
 func (api *API) getSilencesHandler(params silence_ops.GetSilencesParams) middleware.Responder {
+	logger := api.requestLogger(params.HTTPRequest)
+
 	matchers := []*labels.Matcher{}
 	if params.Filter != nil {
 		for _, matcherString := range params.Filter {
-			matcher, err := parse.Matcher(matcherString)
+			matcher, err := labels.ParseMatcher(matcherString)
 			if err != nil {
-				level.Error(api.logger).Log("msg", "failed to parse matchers", "err", err)
+				level.Debug(logger).Log("msg", "Failed to parse matchers", "err", err)
 				return alert_ops.NewGetAlertsBadRequest().WithPayload(err.Error())
 			}
 
@@ -569,42 +520,40 @@ func (api *API) getSilencesHandler(params silence_ops.GetSilencesParams) middlew
 
 	psils, _, err := api.silences.Query()
 	if err != nil {
-		level.Error(api.logger).Log("msg", "failed to get silences", "err", err)
+		level.Error(logger).Log("msg", "Failed to get silences", "err", err)
 		return silence_ops.NewGetSilencesInternalServerError().WithPayload(err.Error())
 	}
 
 	sils := open_api_models.GettableSilences{}
 	for _, ps := range psils {
-		silence, err := gettableSilenceFromProto(ps)
-		if err != nil {
-			level.Error(api.logger).Log("msg", "failed to unmarshal silence from proto", "err", err)
-			return silence_ops.NewGetSilencesInternalServerError().WithPayload(err.Error())
-		}
-		if !gettableSilenceMatchesFilterLabels(silence, matchers) {
+		if !CheckSilenceMatchesFilterLabels(ps, matchers) {
 			continue
 		}
+		silence, err := GettableSilenceFromProto(ps)
+		if err != nil {
+			level.Error(logger).Log("msg", "Failed to unmarshal silence from proto", "err", err)
+			return silence_ops.NewGetSilencesInternalServerError().WithPayload(err.Error())
+		}
 		sils = append(sils, &silence)
 	}
 
-	sortSilences(sils)
+	SortSilences(sils)
 
 	return silence_ops.NewGetSilencesOK().WithPayload(sils)
 }
 
-var (
-	silenceStateOrder = map[types.SilenceState]int{
-		types.SilenceStateActive:  1,
-		types.SilenceStatePending: 2,
-		types.SilenceStateExpired: 3,
-	}
-)
+var silenceStateOrder = map[types.SilenceState]int{
+	types.SilenceStateActive:  1,
+	types.SilenceStatePending: 2,
+	types.SilenceStateExpired: 3,
+}
 
-// sortSilences sorts first according to the state "active, pending, expired"
+// SortSilences sorts first according to the state "active, pending, expired"
 // then by end time or start time depending on the state.
 // active silences should show the next to expire first
 // pending silences are ordered based on which one starts next
 // expired are ordered based on which one expired most recently
-func sortSilences(sils open_api_models.GettableSilences) {
+func SortSilences(sils open_api_models.GettableSilences) {
 	sort.Slice(sils, func(i, j int) bool {
 		state1 := types.SilenceState(*sils[i].Status.State)
 		state2 := types.SilenceState(*sils[j].Status.State)
@@ -629,32 +578,50 @@ func sortSilences(sils open_api_models.GettableSilences) {
 	})
 }
 
-func gettableSilenceMatchesFilterLabels(s open_api_models.GettableSilence, matchers []*labels.Matcher) bool {
-	sms := make(map[string]string)
-	for _, m := range s.Matchers {
-		sms[*m.Name] = *m.Value
+// CheckSilenceMatchesFilterLabels returns true if
+// a given silence matches a list of matchers.
+// A silence matches a filter (list of matchers) if
+// for all matchers in the filter, there exists a matcher in the silence
+// such that their names, types, and values are equivalent.
+func CheckSilenceMatchesFilterLabels(s *silencepb.Silence, matchers []*labels.Matcher) bool {
+	for _, matcher := range matchers {
+		found := false
+		for _, m := range s.Matchers {
+			if matcher.Name == m.Name &&
+				(matcher.Type == labels.MatchEqual && m.Type == silencepb.Matcher_EQUAL ||
+					matcher.Type == labels.MatchRegexp && m.Type == silencepb.Matcher_REGEXP ||
+					matcher.Type == labels.MatchNotEqual && m.Type == silencepb.Matcher_NOT_EQUAL ||
+					matcher.Type == labels.MatchNotRegexp && m.Type == silencepb.Matcher_NOT_REGEXP) &&
+				matcher.Value == m.Pattern {
+				found = true
+				break
+			}
+		}
+		if !found {
+			return false
+		}
 	}
 
-	return matchFilterLabels(matchers, sms)
+	return true
 }
 
-// func matchesFilterLabels(labels model.LabelSet, matchers []*labels.Matcher) bool {
-
 func (api *API) getSilenceHandler(params silence_ops.GetSilenceParams) middleware.Responder {
+	logger := api.requestLogger(params.HTTPRequest)
+
 	sils, _, err := api.silences.Query(silence.QIDs(params.SilenceID.String()))
 	if err != nil {
-		level.Error(api.logger).Log("msg", "failed to get silence by id", "err", err)
+		level.Error(logger).Log("msg", "Failed to get silence by id", "err", err, "id", params.SilenceID.String())
 		return silence_ops.NewGetSilenceInternalServerError().WithPayload(err.Error())
 	}
 
 	if len(sils) == 0 {
-		level.Error(api.logger).Log("msg", "failed to find silence", "err", err)
+		level.Error(logger).Log("msg", "Failed to find silence", "err", err, "id", params.SilenceID.String())
 		return silence_ops.NewGetSilenceNotFound()
 	}
 
-	sil, err := gettableSilenceFromProto(sils[0])
+	sil, err := GettableSilenceFromProto(sils[0])
 	if err != nil {
-		level.Error(api.logger).Log("msg", "failed to convert unmarshal from proto", "err", err)
+		level.Error(logger).Log("msg", "Failed to convert unmarshal from proto", "err", err)
 		return silence_ops.NewGetSilenceInternalServerError().WithPayload(err.Error())
 	}
 
@@ -662,84 +629,45 @@ func (api *API) getSilenceHandler(params silence_ops.GetSilenceParams) middlewar
 }
 
 func (api *API) deleteSilenceHandler(params silence_ops.DeleteSilenceParams) middleware.Responder {
-	sid := params.SilenceID.String()
+	logger := api.requestLogger(params.HTTPRequest)
 
+	sid := params.SilenceID.String()
 	if err := api.silences.Expire(sid); err != nil {
-		level.Error(api.logger).Log("msg", "failed to expire silence", "err", err)
+		level.Error(logger).Log("msg", "Failed to expire silence", "err", err)
+		if err == silence.ErrNotFound {
+			return silence_ops.NewDeleteSilenceNotFound()
+		}
 		return silence_ops.NewDeleteSilenceInternalServerError().WithPayload(err.Error())
 	}
 	return silence_ops.NewDeleteSilenceOK()
 }
 
-func gettableSilenceFromProto(s *silencepb.Silence) (open_api_models.GettableSilence, error) {
-	start := strfmt.DateTime(s.StartsAt)
-	end := strfmt.DateTime(s.EndsAt)
-	updated := strfmt.DateTime(s.UpdatedAt)
-	state := string(types.CalcSilenceState(s.StartsAt, s.EndsAt))
-	sil := open_api_models.GettableSilence{
-		Silence: open_api_models.Silence{
-			StartsAt:  &start,
-			EndsAt:    &end,
-			Comment:   &s.Comment,
-			CreatedBy: &s.CreatedBy,
-		},
-		ID:        &s.Id,
-		UpdatedAt: &updated,
-		Status: &open_api_models.SilenceStatus{
-			State: &state,
-		},
-	}
-
-	for _, m := range s.Matchers {
-		matcher := &open_api_models.Matcher{
-			Name:  &m.Name,
-			Value: &m.Pattern,
-		}
-		switch m.Type {
-		case silencepb.Matcher_EQUAL:
-			f := false
-			matcher.IsRegex = &f
-		case silencepb.Matcher_REGEXP:
-			t := true
-			matcher.IsRegex = &t
-		default:
-			return sil, fmt.Errorf(
-				"unknown matcher type for matcher '%v' in silence '%v'",
-				m.Name,
-				s.Id,
-			)
-		}
-		sil.Matchers = append(sil.Matchers, matcher)
-	}
-
-	return sil, nil
-}
-
 func (api *API) postSilencesHandler(params silence_ops.PostSilencesParams) middleware.Responder {
+	logger := api.requestLogger(params.HTTPRequest)
 
-	sil, err := postableSilenceToProto(params.Silence)
+	sil, err := PostableSilenceToProto(params.Silence)
 	if err != nil {
-		level.Error(api.logger).Log("msg", "failed to marshal silence to proto", "err", err)
+		level.Error(logger).Log("msg", "Failed to marshal silence to proto", "err", err)
 		return silence_ops.NewPostSilencesBadRequest().WithPayload(
 			fmt.Sprintf("failed to convert API silence to internal silence: %v", err.Error()),
 		)
 	}
 
 	if sil.StartsAt.After(sil.EndsAt) || sil.StartsAt.Equal(sil.EndsAt) {
-		msg := "failed to create silence: start time must be equal or after end time"
-		level.Error(api.logger).Log("msg", msg, "err", err)
+		msg := "Failed to create silence: start time must be before end time"
+		level.Error(logger).Log("msg", msg, "starts_at", sil.StartsAt, "ends_at", sil.EndsAt)
 		return silence_ops.NewPostSilencesBadRequest().WithPayload(msg)
 	}
 
 	if sil.EndsAt.Before(time.Now()) {
-		msg := "failed to create silence: end time can't be in the past"
-		level.Error(api.logger).Log("msg", msg, "err", err)
+		msg := "Failed to create silence: end time can't be in the past"
+		level.Error(logger).Log("msg", msg, "ends_at", sil.EndsAt)
 		return silence_ops.NewPostSilencesBadRequest().WithPayload(msg)
 	}
 
 	sid, err := api.silences.Set(sil)
 	if err != nil {
-		level.Error(api.logger).Log("msg", "failed to create silence", "err", err)
+		level.Error(logger).Log("msg", "Failed to create silence", "err", err)
 		if err == silence.ErrNotFound {
 			return silence_ops.NewPostSilencesNotFound().WithPayload(err.Error())
 		}
@@ -751,32 +679,10 @@ func (api *API) postSilencesHandler(params silence_ops.PostSilencesParams) middl
 	})
 }
 
-func postableSilenceToProto(s *open_api_models.PostableSilence) (*silencepb.Silence, error) {
-	sil := &silencepb.Silence{
-		Id:        s.ID,
-		StartsAt:  time.Time(*s.StartsAt),
-		EndsAt:    time.Time(*s.EndsAt),
-		Comment:   *s.Comment,
-		CreatedBy: *s.CreatedBy,
-	}
-	for _, m := range s.Matchers {
-		matcher := &silencepb.Matcher{
-			Name:    *m.Name,
-			Pattern: *m.Value,
-			Type:    silencepb.Matcher_EQUAL,
-		}
-		if *m.IsRegex {
-			matcher.Type = silencepb.Matcher_REGEXP
-		}
-		sil.Matchers = append(sil.Matchers, matcher)
-	}
-	return sil, nil
-}
-
 func parseFilter(filter []string) ([]*labels.Matcher, error) {
 	matchers := make([]*labels.Matcher, 0, len(filter))
 	for _, matcherString := range filter {
-		matcher, err := parse.Matcher(matcherString)
+		matcher, err := labels.ParseMatcher(matcherString)
 		if err != nil {
 			return nil, err
 		}
@@ -785,3 +691,33 @@ func parseFilter(filter []string) ([]*labels.Matcher, error) {
 	}
 	return matchers, nil
 }
+
+var (
+	swaggerSpecCacheMx       sync.Mutex
+	swaggerSpecCache         *loads.Document
+	swaggerSpecAnalysisCache *analysis.Spec
+)
+
+// getSwaggerSpec loads and caches the swagger spec. If a cached version already exists,
+// it returns the cached one. The reason why we cache it is because some downstream projects
+// (e.g. Grafana Mimir) creates many Alertmanager instances in the same process, so they would
+// incur in a significant memory penalty if we would reload the swagger spec each time.
+func getSwaggerSpec() (*loads.Document, *analysis.Spec, error) {
+	swaggerSpecCacheMx.Lock()
+	defer swaggerSpecCacheMx.Unlock()
+
+	// Check if a cached version exists.
+	if swaggerSpecCache != nil {
+		return swaggerSpecCache, swaggerSpecAnalysisCache, nil
+	}
+
+	// Load embedded swagger file.
+	swaggerSpec, err := loads.Analyzed(restapi.SwaggerJSON, "")
+	if err != nil {
+		return nil, nil, fmt.Errorf("failed to load embedded swagger file: %w", err)
+	}
+
+	swaggerSpecCache = swaggerSpec
+	swaggerSpecAnalysisCache = analysis.New(swaggerSpec.Spec())
+	return swaggerSpec, swaggerSpecAnalysisCache, nil
+}
